@@ -4,21 +4,22 @@ use std::cell::UnsafeCell;
 use std::collections::{HashMap, HashSet};
 use std::ffi::{OsStr, OsString};
 use std::fmt;
-use std::fs::{self, DirEntry, File};
+use std::fs::{self, DirEntry, File, read_dir};
 use std::io::Read;
 use std::os::unix::ffi::OsStrExt;
 use std::path::{Path, PathBuf};
+use std::process::ExitStatus;
 use std::str::{self, FromStr};
 use std::sync::atomic::{AtomicUsize, Ordering};
 
-use libc::{c_ulong, gid_t, kill, uid_t};
+use libc::{c_ulong, gid_t, uid_t};
 
 use crate::sys::system::SystemInfo;
-use crate::sys::utils::{
-    get_all_data_from_file, get_all_utf8_data, realpath, PathHandler, PathPush,
-};
+use crate::sys::utils::{PathHandler, PathPush, get_all_data_from_file, get_all_utf8_data};
+use crate::unix::utils::realpath;
 use crate::{
-    DiskUsage, Gid, Pid, Process, ProcessesToUpdate, ProcessRefreshKind, ProcessStatus, Signal, ThreadKind, Uid,
+    DiskUsage, Gid, Pid, Process, ProcessRefreshKind, ProcessStatus, ProcessesToUpdate, Signal,
+    ThreadKind, Uid,
 };
 
 use crate::sys::system::remaining_files;
@@ -108,6 +109,7 @@ pub(crate) struct ProcessInner {
     old_stime: u64,
     start_time_without_boot_time: u64,
     start_time: u64,
+    start_time_raw: u64,
     run_time: u64,
     pub(crate) updated: bool,
     cpu_usage: f32,
@@ -124,6 +126,8 @@ pub(crate) struct ProcessInner {
     written_bytes: u64,
     thread_kind: Option<ThreadKind>,
     proc_path: PathBuf,
+    accumulated_cpu_time: u64,
+    exists: bool,
 }
 
 impl ProcessInner {
@@ -147,6 +151,7 @@ impl ProcessInner {
             updated: true,
             start_time_without_boot_time: 0,
             start_time: 0,
+            start_time_raw: 0,
             run_time: 0,
             user_id: None,
             effective_user_id: None,
@@ -161,12 +166,14 @@ impl ProcessInner {
             written_bytes: 0,
             thread_kind: None,
             proc_path,
+            accumulated_cpu_time: 0,
+            exists: true,
         }
     }
 
     pub(crate) fn kill_with(&self, signal: Signal) -> Option<bool> {
         let c_signal = crate::sys::system::convert_signal(signal)?;
-        unsafe { Some(kill(self.pid.0, c_signal) == 0) }
+        unsafe { Some(libc::kill(self.pid.0, c_signal) == 0) }
     }
 
     pub(crate) fn name(&self) -> &OsStr {
@@ -225,6 +232,10 @@ impl ProcessInner {
         self.cpu_usage
     }
 
+    pub(crate) fn accumulated_cpu_time(&self) -> u64 {
+        self.accumulated_cpu_time
+    }
+
     pub(crate) fn disk_usage(&self) -> DiskUsage {
         DiskUsage {
             written_bytes: self.written_bytes.saturating_sub(self.old_written_bytes),
@@ -250,18 +261,17 @@ impl ProcessInner {
         self.effective_group_id
     }
 
-    pub(crate) fn wait(&self) {
-        let mut status = 0;
-        // attempt waiting
-        unsafe {
-            if retry_eintr!(libc::waitpid(self.pid.0, &mut status, 0)) < 0 {
-                // attempt failed (non-child process) so loop until process ends
-                let duration = std::time::Duration::from_millis(10);
-                while kill(self.pid.0, 0) == 0 {
-                    std::thread::sleep(duration);
-                }
-            }
+    pub(crate) fn wait(&self) -> Option<ExitStatus> {
+        // If anything fails when trying to retrieve the start time, better to return `None`.
+        let (data, _) = _get_stat_data_and_file(&self.proc_path).ok()?;
+        let parts = parse_stat_file(&data)?;
+
+        if start_time_raw(&parts) != self.start_time_raw {
+            sysinfo_debug!("Seems to not be the same process anymore");
+            return None;
         }
+
+        crate::unix::utils::wait_process(self.pid)
     }
 
     pub(crate) fn session_id(&self) -> Option<Pid> {
@@ -280,7 +290,52 @@ impl ProcessInner {
     }
 
     pub(crate) fn switch_updated(&mut self) -> bool {
-         std::mem::replace(&mut self.updated, false)
+        std::mem::replace(&mut self.updated, false)
+    }
+
+    pub(crate) fn set_nonexistent(&mut self) {
+        self.exists = false;
+    }
+
+    pub(crate) fn exists(&self) -> bool {
+        self.exists
+    }
+
+    pub(crate) fn open_files(&self) -> Option<usize> {
+        let open_files_dir = self.proc_path.as_path().join("fd");
+        match fs::read_dir(&open_files_dir) {
+            Ok(entries) => Some(entries.count() as _),
+            Err(_error) => {
+                sysinfo_debug!(
+                    "Failed to get open files in `{}`: {_error:?}",
+                    open_files_dir.display(),
+                );
+                None
+            }
+        }
+    }
+
+    pub(crate) fn open_files_limit(&self) -> Option<usize> {
+        let limits_files = self.proc_path.as_path().join("limits");
+        match fs::read_to_string(&limits_files) {
+            Ok(content) => {
+                for line in content.lines() {
+                    if let Some(line) = line.strip_prefix("Max open files ")
+                        && let Some(nb) = line.split_whitespace().find(|p| !p.is_empty())
+                    {
+                        return usize::from_str(nb).ok();
+                    }
+                }
+                None
+            }
+            Err(_error) => {
+                sysinfo_debug!(
+                    "Failed to get limits in `{}`: {_error:?}",
+                    limits_files.display()
+                );
+                None
+            }
+        }
     }
 }
 
@@ -306,11 +361,10 @@ pub(crate) fn set_time(p: &mut ProcessInner, utime: u64, stime: u64) {
     p.old_stime = p.stime;
     p.utime = utime;
     p.stime = stime;
-    p.updated = true;
 }
 
 pub(crate) fn update_process_disk_activity(p: &mut ProcessInner, path: &mut PathHandler) {
-    let data = match get_all_utf8_data(path.join("io"), 16_384) {
+    let data = match get_all_utf8_data(path.replace_and_join("io"), 16_384) {
         Ok(d) => d,
         Err(_) => return,
     };
@@ -355,15 +409,26 @@ unsafe impl<T> Send for Wrap<'_, T> {}
 unsafe impl<T> Sync for Wrap<'_, T> {}
 
 #[inline(always)]
-fn compute_start_time_without_boot_time(parts: &Parts<'_>, info: &SystemInfo) -> u64 {
+fn start_time_raw(parts: &Parts<'_>) -> u64 {
+    u64::from_str(parts.str_parts[ProcIndex::StartTime as usize]).unwrap_or(0)
+}
+
+#[inline(always)]
+fn compute_start_time_without_boot_time(parts: &Parts<'_>, info: &SystemInfo) -> (u64, u64) {
+    let raw = start_time_raw(parts);
     // To be noted that the start time is invalid here, it still needs to be converted into
     // "real" time.
-    u64::from_str(parts.str_parts[ProcIndex::StartTime as usize]).unwrap_or(0) / info.clock_cycle
+    (raw, raw / info.clock_cycle)
+}
+
+fn _get_stat_data_and_file(path: &Path) -> Result<(Vec<u8>, File), ()> {
+    let mut file = File::open(path.join("stat")).map_err(|_| ())?;
+    let data = get_all_data_from_file(&mut file, 1024).map_err(|_| ())?;
+    Ok((data, file))
 }
 
 fn _get_stat_data(path: &Path, stat_file: &mut Option<FileCounter>) -> Result<Vec<u8>, ()> {
-    let mut file = File::open(path.join("stat")).map_err(|_| ())?;
-    let data = get_all_data_from_file(&mut file, 1024).map_err(|_| ())?;
+    let (data, file) = _get_stat_data_and_file(path)?;
     *stat_file = FileCounter::new(file);
     Ok(data)
 }
@@ -387,7 +452,7 @@ fn refresh_user_group_ids(
     }
 
     if let Some(((user_id, effective_user_id), (group_id, effective_group_id))) =
-        get_uid_and_gid(path.join("status"))
+        get_uid_and_gid(path.replace_and_join("status"))
     {
         p.user_id = Some(Uid(user_id));
         p.effective_user_id = Some(Uid(effective_user_id));
@@ -414,26 +479,48 @@ fn update_proc_info(
     if refresh_kind.exe().needs_update(|| p.exe.is_none()) {
         // Do not use cmd[0] because it is not the same thing.
         // See https://github.com/GuillaumeGomez/sysinfo/issues/697.
-        p.exe = realpath(proc_path.join("exe"));
+        p.exe = realpath(proc_path.replace_and_join("exe"));
+        // If the target executable file was modified or removed, linux appends ` (deleted)` at
+        // the end. We need to remove it.
+        // See https://github.com/GuillaumeGomez/sysinfo/issues/1585.
+        let deleted = b" (deleted)";
+        if let Some(exe) = &mut p.exe
+            && let Some(file_name) = exe.file_name()
+            && file_name.as_encoded_bytes().ends_with(deleted)
+        {
+            let mut file_name = file_name.as_encoded_bytes().to_vec();
+            file_name.truncate(file_name.len() - deleted.len());
+            unsafe {
+                exe.set_file_name(OsString::from_encoded_bytes_unchecked(file_name));
+            }
+        }
     }
 
     if refresh_kind.cmd().needs_update(|| p.cmd.is_empty()) {
-        p.cmd = copy_from_file(proc_path.join("cmdline"));
+        p.cmd = copy_from_file(proc_path.replace_and_join("cmdline"));
     }
     if refresh_kind.environ().needs_update(|| p.environ.is_empty()) {
-        p.environ = copy_from_file(proc_path.join("environ"));
+        p.environ = copy_from_file(proc_path.replace_and_join("environ"));
     }
     if refresh_kind.cwd().needs_update(|| p.cwd.is_none()) {
-        p.cwd = realpath(proc_path.join("cwd"));
+        p.cwd = realpath(proc_path.replace_and_join("cwd"));
     }
     if refresh_kind.root().needs_update(|| p.root.is_none()) {
-        p.root = realpath(proc_path.join("root"));
+        p.root = realpath(proc_path.replace_and_join("root"));
     }
 
     update_time_and_memory(proc_path, p, str_parts, uptime, info, refresh_kind);
     if refresh_kind.disk_usage() {
         update_process_disk_activity(p, proc_path);
     }
+    // Needs to be after `update_time_and_memory`.
+    if refresh_kind.cpu() {
+        // The external values for CPU times are in "ticks", which are
+        // scaled by "HZ", which is pegged externally at 100 ticks/second.
+        p.accumulated_cpu_time =
+            p.utime.saturating_add(p.stime).saturating_mul(1_000) / info.clock_cycle;
+    }
+    p.updated = true;
 }
 
 fn update_parent_pid(p: &mut ProcessInner, parent_pid: Option<Pid>, str_parts: &[&str]) {
@@ -459,7 +546,10 @@ fn retrieve_all_new_process_info(
     let mut proc_path = PathHandler::new(path);
     let name = parts.short_exe;
 
-    p.start_time_without_boot_time = compute_start_time_without_boot_time(parts, info);
+    let (start_time_raw, start_time_without_boot_time) =
+        compute_start_time_without_boot_time(parts, info);
+    p.start_time_raw = start_time_raw;
+    p.start_time_without_boot_time = start_time_without_boot_time;
     p.start_time = p
         .start_time_without_boot_time
         .saturating_add(info.boot_time);
@@ -487,6 +577,71 @@ fn retrieve_all_new_process_info(
     Process { inner: p }
 }
 
+fn update_existing_process(
+    proc: &mut Process,
+    parent_pid: Option<Pid>,
+    uptime: u64,
+    info: &SystemInfo,
+    refresh_kind: ProcessRefreshKind,
+    tasks: Option<HashSet<Pid>>,
+) -> Result<Option<Process>, ()> {
+    let entry = &mut proc.inner;
+    let data = if let Some(mut f) = entry.stat_file.take() {
+        match get_all_data_from_file(&mut f, 1024) {
+            Ok(data) => {
+                // Everything went fine, we put back the file descriptor.
+                entry.stat_file = Some(f);
+                data
+            }
+            Err(_) => {
+                // It's possible that the file descriptor is no longer valid in case the
+                // original process was terminated and another one took its place.
+                _get_stat_data(&entry.proc_path, &mut entry.stat_file)?
+            }
+        }
+    } else {
+        _get_stat_data(&entry.proc_path, &mut entry.stat_file)?
+    };
+    entry.tasks = tasks;
+
+    let parts = parse_stat_file(&data).ok_or(())?;
+    let start_time_raw = start_time_raw(&parts);
+
+    // It's possible that a new process took this same PID when the "original one" terminated.
+    // If the start time differs, then it means it's not the same process anymore and that we
+    // need to get all its information, hence why we check it here.
+    if start_time_raw == entry.start_time_raw {
+        let mut proc_path = PathHandler::new(&entry.proc_path);
+
+        update_proc_info(
+            entry,
+            parent_pid,
+            refresh_kind,
+            &mut proc_path,
+            &parts.str_parts,
+            uptime,
+            info,
+        );
+
+        refresh_user_group_ids(entry, &mut proc_path, refresh_kind);
+        return Ok(None);
+    }
+    // If we're here, it means that the PID still exists but it's a different process.
+    let p = retrieve_all_new_process_info(
+        entry.pid,
+        parent_pid,
+        &parts,
+        &entry.proc_path,
+        info,
+        refresh_kind,
+        uptime,
+    );
+    *proc = p;
+    // Since this PID is already in the HashMap, no need to add it again.
+    Ok(None)
+}
+
+#[allow(clippy::too_many_arguments)]
 pub(crate) fn _get_process_data(
     path: &Path,
     proc_list: &mut HashMap<Pid, Process>,
@@ -495,78 +650,20 @@ pub(crate) fn _get_process_data(
     uptime: u64,
     info: &SystemInfo,
     refresh_kind: ProcessRefreshKind,
-) -> Result<(Option<Process>, Pid), ()> {
-    let data;
-    let parts = if let Some(ref mut entry) = proc_list.get_mut(&pid) {
-        let entry = &mut entry.inner;
-        data = if let Some(mut f) = entry.stat_file.take() {
-            match get_all_data_from_file(&mut f, 1024) {
-                Ok(data) => {
-                    // Everything went fine, we put back the file descriptor.
-                    entry.stat_file = Some(f);
-                    data
-                }
-                Err(_) => {
-                    // It's possible that the file descriptor is no longer valid in case the
-                    // original process was terminated and another one took its place.
-                    _get_stat_data(&entry.proc_path, &mut entry.stat_file)?
-                }
-            }
-        } else {
-            _get_stat_data(path, &mut entry.stat_file)?
-        };
-        let parts = parse_stat_file(&data).ok_or(())?;
-        let start_time_without_boot_time = compute_start_time_without_boot_time(&parts, info);
-
-        // It's possible that a new process took this same PID when the "original one" terminated.
-        // If the start time differs, then it means it's not the same process anymore and that we
-        // need to get all its information, hence why we check it here.
-        if start_time_without_boot_time == entry.start_time_without_boot_time {
-            let mut proc_path = PathHandler::new(path);
-
-            update_proc_info(
-                entry,
-                parent_pid,
-                refresh_kind,
-                &mut proc_path,
-                &parts.str_parts,
-                uptime,
-                info,
-            );
-
-            refresh_user_group_ids(entry, &mut proc_path, refresh_kind);
-            return Ok((None, pid));
-        }
-        parts
-    } else {
-        let mut stat_file = None;
-        let data = _get_stat_data(path, &mut stat_file)?;
-        let parts = parse_stat_file(&data).ok_or(())?;
-
-        let mut p = retrieve_all_new_process_info(
-            pid,
-            parent_pid,
-            &parts,
-            path,
-            info,
-            refresh_kind,
-            uptime,
-        );
-        p.inner.stat_file = stat_file;
-        return Ok((Some(p), pid));
-    };
-
-    // If we're here, it means that the PID still exists but it's a different process.
-    let p =
-        retrieve_all_new_process_info(pid, parent_pid, &parts, path, info, refresh_kind, uptime);
-    match proc_list.get_mut(&pid) {
-        Some(ref mut entry) => **entry = p,
-        // If it ever enters this case, it means that the process was removed from the HashMap
-        // in-between with the usage of dark magic.
-        None => unreachable!(),
+    tasks: Option<HashSet<Pid>>,
+) -> Result<Option<Process>, ()> {
+    if let Some(ref mut entry) = proc_list.get_mut(&pid) {
+        return update_existing_process(entry, parent_pid, uptime, info, refresh_kind, tasks);
     }
-    // Since this PID is already in the HashMap, no need to add it again.
-    Ok((None, pid))
+    let mut stat_file = None;
+    let data = _get_stat_data(path, &mut stat_file)?;
+    let parts = parse_stat_file(&data).ok_or(())?;
+
+    let mut new_process =
+        retrieve_all_new_process_info(pid, parent_pid, &parts, path, info, refresh_kind, uptime);
+    new_process.inner.stat_file = stat_file;
+    new_process.inner.tasks = tasks;
+    Ok(Some(new_process))
 }
 
 fn old_get_memory(entry: &mut ProcessInner, str_parts: &[&str], info: &SystemInfo) {
@@ -634,7 +731,7 @@ fn update_time_and_memory(
         #[allow(clippy::collapsible_if)]
         if refresh_kind.memory() {
             // Keeping this nested level for readability reasons.
-            if !get_memory(path.join("statm"), entry, info) {
+            if !get_memory(path.replace_and_join("statm"), entry, info) {
                 old_get_memory(entry, str_parts, info);
             }
         }
@@ -654,53 +751,6 @@ struct ProcAndTasks {
     tasks: Option<HashSet<Pid>>,
 }
 
-fn get_all_pid_entries(
-    parent: Option<&OsStr>,
-    parent_pid: Option<Pid>,
-    entry: DirEntry,
-    data: &mut Vec<ProcAndTasks>,
-) -> Option<Pid> {
-    let Ok(file_type) = entry.file_type() else {
-        return None;
-    };
-    if !file_type.is_dir() {
-        return None;
-    }
-
-    let entry = entry.path();
-    let name = entry.file_name();
-
-    if name == parent {
-        // Needed because tasks have their own PID listed in the "task" folder.
-        return None;
-    }
-    let name = name?;
-    let pid = Pid::from(usize::from_str(name.to_str()?).ok()?);
-
-    let tasks_dir = Path::join(&entry, "task");
-
-    let tasks = if let Ok(entries) = fs::read_dir(tasks_dir) {
-        let mut tasks = HashSet::new();
-        for task in entries
-            .into_iter()
-            .filter_map(|entry| get_all_pid_entries(Some(name), Some(pid), entry.ok()?, data))
-        {
-            tasks.insert(task);
-        }
-        Some(tasks)
-    } else {
-        None
-    };
-
-    data.push(ProcAndTasks {
-        pid,
-        parent_pid,
-        path: entry,
-        tasks,
-    });
-    Some(pid)
-}
-
 #[cfg(feature = "multithread")]
 #[inline]
 pub(crate) fn iter<T>(val: T) -> rayon::iter::IterBridge<T>
@@ -719,9 +769,11 @@ where
     val
 }
 
+/// We're forced to read the whole `/proc` folder because if a process died and another took its
+/// place, we need to get the task parent (if it's a task).
 pub(crate) fn refresh_procs(
     proc_list: &mut HashMap<Pid, Process>,
-    path: &Path,
+    proc_path: &Path,
     uptime: u64,
     info: &SystemInfo,
     processes_to_update: ProcessesToUpdate<'_>,
@@ -730,69 +782,53 @@ pub(crate) fn refresh_procs(
     #[cfg(feature = "multithread")]
     use rayon::iter::ParallelIterator;
 
-    #[inline(always)]
-    fn real_filter(e: &ProcAndTasks, filter: &[Pid]) -> bool {
-        filter.contains(&e.pid)
-    }
-
-    #[inline(always)]
-    fn empty_filter(_e: &ProcAndTasks, _filter: &[Pid]) -> bool {
-        true
-    }
-
-    #[allow(clippy::type_complexity)]
-    let (filter, filter_callback): (
-        &[Pid],
-        &(dyn Fn(&ProcAndTasks, &[Pid]) -> bool + Sync + Send),
-    ) = match processes_to_update {
-        ProcessesToUpdate::All => (&[], &empty_filter),
-        ProcessesToUpdate::Some(pids) => {
-            if pids.is_empty() {
-                return 0;
-            }
-            (pids, &real_filter)
-        }
-    };
-
     let nb_updated = AtomicUsize::new(0);
 
-    // FIXME: To prevent retrieving a task more than once (it can be listed in `/proc/[PID]/task`
-    // subfolder and directly in `/proc` at the same time), might be interesting to use a `HashSet`.
+    // This code goes through processes (listed in `/proc`) and through tasks (listed in
+    // `/proc/[PID]/task`). However, the stored tasks information is supposed to be already present
+    // in the PIDs listed from `/proc` so there will be no duplicates between PIDs and tasks PID.
+    //
+    // If a task is not listed in `/proc`, then we don't retrieve its information.
+    //
+    // So in short: since we update the `HashMap` itself by adding/removing entries outside of the
+    // parallel iterator, we can safely use it inside the parallel iterator and update its entries
+    // concurrently.
     let procs = {
-        let d = match fs::read_dir(path) {
-            Ok(d) => d,
-            Err(_err) => {
-                sysinfo_debug!("Failed to read folder {path:?}: {_err:?}");
-                return 0
+        let pid_iter: Box<dyn Iterator<Item = (PathBuf, Pid)> + Send> = match processes_to_update {
+            ProcessesToUpdate::All => match read_dir(proc_path) {
+                Ok(proc_entries) => Box::new(proc_entries.filter_map(filter_pid_entries)),
+                Err(_err) => {
+                    sysinfo_debug!("Failed to read folder {proc_path:?}: {_err:?}");
+                    return 0;
+                }
             },
+            ProcessesToUpdate::Some(pids) => Box::new(
+                pids.iter()
+                    .map(|pid| (proc_path.join(pid.to_string()), *pid)),
+            ),
         };
+
         let proc_list = Wrap(UnsafeCell::new(proc_list));
 
-        iter(d)
-            .map(|entry| {
-                let Ok(entry) = entry else { return Vec::new() };
-                let mut entries = Vec::new();
-                get_all_pid_entries(None, None, entry, &mut entries);
-                entries
+        iter(pid_iter)
+            .flat_map(|(path, pid)| {
+                get_proc_and_tasks(path, pid, refresh_kind, processes_to_update)
             })
-            .flatten()
-            .filter(|e| filter_callback(e, filter))
             .filter_map(|e| {
-                let (mut p, _) = _get_process_data(
+                let proc_list = proc_list.get();
+                let new_process = _get_process_data(
                     e.path.as_path(),
-                    proc_list.get(),
+                    proc_list,
                     e.pid,
                     e.parent_pid,
                     uptime,
                     info,
                     refresh_kind,
+                    e.tasks,
                 )
                 .ok()?;
                 nb_updated.fetch_add(1, Ordering::Relaxed);
-                if let Some(ref mut p) = p {
-                    p.inner.tasks = e.tasks;
-                }
-                p
+                new_process
             })
             .collect::<Vec<_>>()
     };
@@ -802,24 +838,95 @@ pub(crate) fn refresh_procs(
     nb_updated.into_inner()
 }
 
-// FIXME: To be removed once MSRV for this crate is 1.80 nd use the `trim_ascii()` method instead.
-fn trim_ascii(mut bytes: &[u8]) -> &[u8] {
-    // Code from Rust code library.
-    while let [rest @ .., last] = bytes {
-        if last.is_ascii_whitespace() {
-            bytes = rest;
-        } else {
-            break;
+fn filter_pid_entries(entry: Result<DirEntry, std::io::Error>) -> Option<(PathBuf, Pid)> {
+    if let Ok(entry) = entry
+        && let Ok(file_type) = entry.file_type()
+        && file_type.is_dir()
+        && let Some(name) = entry.file_name().to_str()
+        && let Ok(pid) = usize::from_str(name)
+    {
+        Some((entry.path(), Pid::from(pid)))
+    } else {
+        None
+    }
+}
+
+fn get_proc_and_tasks(
+    path: PathBuf,
+    pid: Pid,
+    refresh_kind: ProcessRefreshKind,
+    processes_to_update: ProcessesToUpdate<'_>,
+) -> Vec<ProcAndTasks> {
+    let mut parent_pid = None;
+    let (mut procs, mut tasks) = if refresh_kind.tasks() {
+        let procs = get_proc_tasks(&path, pid);
+        let tasks = procs.iter().map(|ProcAndTasks { pid, .. }| *pid).collect();
+
+        (procs, Some(tasks))
+    } else {
+        (Vec::new(), None)
+    };
+
+    if processes_to_update != ProcessesToUpdate::All {
+        // If the process' tgid doesn't match its pid, it is a task
+        if let Some(tgid) = get_tgid(&path.join("status"))
+            && tgid != pid
+        {
+            parent_pid = Some(tgid);
+            tasks = None;
+        }
+
+        // Don't add the tasks to the list of processes to update
+        procs.clear();
+    }
+
+    procs.push(ProcAndTasks {
+        pid,
+        parent_pid,
+        path,
+        tasks,
+    });
+
+    procs
+}
+
+fn get_proc_tasks(path: &Path, parent_pid: Pid) -> Vec<ProcAndTasks> {
+    let task_path = path.join("task");
+
+    read_dir(task_path)
+        .ok()
+        .map(|task_entries| {
+            task_entries
+                .filter_map(filter_pid_entries)
+                // Needed because tasks have their own PID listed in the "task" folder.
+                .filter(|(_, pid)| *pid != parent_pid)
+                .map(|(path, pid)| ProcAndTasks {
+                    pid,
+                    path,
+                    parent_pid: Some(parent_pid),
+                    tasks: None,
+                })
+                .collect()
+        })
+        .unwrap_or_default()
+}
+
+fn split_content(mut data: &[u8]) -> Vec<OsString> {
+    let mut out = Vec::with_capacity(10);
+    while let Some(pos) = data.iter().position(|c| *c == 0) {
+        let s = &data[..pos].trim_ascii();
+        if !s.is_empty() {
+            out.push(OsStr::from_bytes(s).to_os_string());
+        }
+        data = &data[pos + 1..];
+    }
+    if !data.is_empty() {
+        let s = data.trim_ascii();
+        if !s.is_empty() {
+            out.push(OsStr::from_bytes(s).to_os_string());
         }
     }
-    while let [first, rest @ ..] = bytes {
-        if first.is_ascii_whitespace() {
-            bytes = rest;
-        } else {
-            break;
-        }
-    }
-    bytes
+    out
 }
 
 fn copy_from_file(entry: &Path) -> Vec<OsString> {
@@ -831,16 +938,7 @@ fn copy_from_file(entry: &Path) -> Vec<OsString> {
                 sysinfo_debug!("Failed to read file in `copy_from_file`: {:?}", _e);
                 Vec::new()
             } else {
-                let mut out = Vec::with_capacity(10);
-                let mut data = data.as_slice();
-                while let Some(pos) = data.iter().position(|c| *c == 0) {
-                    let s = trim_ascii(&data[..pos]);
-                    if !s.is_empty() {
-                        out.push(OsStr::from_bytes(s).to_os_string());
-                    }
-                    data = &data[pos + 1..];
-                }
-                out
+                split_content(&data)
             }
         }
         Err(_e) => {
@@ -897,6 +995,15 @@ fn get_uid_and_gid(file_path: &Path) -> Option<((uid_t, uid_t), (gid_t, gid_t))>
     }
 }
 
+fn get_tgid(file_path: &Path) -> Option<Pid> {
+    const TGID_KEY: &str = "Tgid:";
+    let status_data = get_all_utf8_data(file_path, 16_385).ok()?;
+    let tgid_line = status_data
+        .lines()
+        .find(|line| line.starts_with(TGID_KEY))?;
+    tgid_line[TGID_KEY.len()..].trim_start().parse().ok()
+}
+
 struct Parts<'a> {
     str_parts: Vec<&'a str>,
     short_exe: &'a [u8],
@@ -962,3 +1069,24 @@ impl Drop for FileCounter {
         remaining_files().fetch_add(1, Ordering::Relaxed);
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use super::split_content;
+    use std::ffi::OsString;
+
+    // This test ensures that all the parts of the data are split.
+    #[test]
+    fn test_copy_file() {
+        assert_eq!(split_content(b"hello\0"), vec![OsString::from("hello")]);
+        assert_eq!(split_content(b"hello"), vec![OsString::from("hello")]);
+        assert_eq!(
+            split_content(b"hello\0b"),
+            vec![OsString::from("hello"), "b".into()]
+        );
+        assert_eq!(
+            split_content(b"hello\0\0\0\0b"),
+            vec![OsString::from("hello"), "b".into()]
+        );
+    }
+}
