@@ -1,18 +1,31 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::{Disk, DiskKind};
+use crate::{Disk, DiskKind, DiskRefreshKind, DiskUsage};
 
 use std::{ffi::OsStr, path::Path};
 
-pub(crate) struct DiskInner;
+pub(crate) struct DiskInner {
+    #[cfg(test)]
+    #[allow(dead_code)]
+    pub(crate) file_system: std::ffi::OsString,
+}
+
+#[cfg(test)]
+impl Default for DiskInner {
+    fn default() -> Self {
+        Self {
+            file_system: std::ffi::OsString::new(),
+        }
+    }
+}
 
 impl DiskInner {
     pub(crate) fn kind(&self) -> DiskKind {
-        unreachable!()
+        DiskKind::Unknown(-1)
     }
 
     pub(crate) fn name(&self) -> &OsStr {
-        unreachable!()
+        OsStr::new("")
     }
 
     pub(crate) fn file_system(&self) -> &OsStr {
@@ -39,9 +52,13 @@ impl DiskInner {
         false
     }
 
-    pub(crate) fn refresh(&mut self) -> bool {
+    pub(crate) fn refresh_specifics(&mut self, _refreshes: DiskRefreshKind) -> bool {
         true
     }
+
+    pub(crate) fn usage(&self) -> DiskUsage {
+        DiskUsage::default()
+    }
 }
 
 pub(crate) struct DisksInner {
@@ -61,7 +78,11 @@ impl DisksInner {
         self.disks
     }
 
-    pub(crate) fn refresh_list(&mut self) {
+    pub(crate) fn refresh_specifics(
+        &mut self,
+        _remove_not_listed_disks: bool,
+        _refreshes: DiskRefreshKind,
+    ) {
         // Does nothing.
     }
 
