@@ -9,43 +9,51 @@ use std::ffi::{OsStr, OsString};
 use std::fmt;
 #[cfg(feature = "debug")]
 use std::io;
-use std::mem::{size_of, zeroed, MaybeUninit};
+use std::mem::{MaybeUninit, size_of, zeroed};
 use std::os::windows::ffi::OsStringExt;
-use std::os::windows::process::CommandExt;
+use std::os::windows::process::{CommandExt, ExitStatusExt};
 use std::path::{Path, PathBuf};
-use std::process;
+use std::process::{self, ExitStatus};
 use std::ptr::null_mut;
-use std::str;
+use std::str::{self, FromStr};
 use std::sync::{Arc, OnceLock};
+use std::time::Instant;
 
 use libc::c_void;
-use ntapi::ntexapi::SYSTEM_PROCESS_INFORMATION;
 use ntapi::ntrtl::RTL_USER_PROCESS_PARAMETERS;
 use ntapi::ntwow64::{PEB32, RTL_USER_PROCESS_PARAMETERS32};
-use windows::core::PCWSTR;
 use windows::Wdk::System::SystemServices::RtlGetVersion;
 use windows::Wdk::System::Threading::{
-    NtQueryInformationProcess, ProcessBasicInformation, ProcessCommandLineInformation,
-    ProcessWow64Information, PROCESSINFOCLASS,
+    NtQueryInformationProcess, PROCESSINFOCLASS, ProcessBasicInformation,
+    ProcessCommandLineInformation, ProcessWow64Information,
 };
 use windows::Win32::Foundation::{
-    LocalFree, ERROR_INSUFFICIENT_BUFFER, FILETIME, HANDLE, HINSTANCE, HLOCAL, MAX_PATH,
+    ERROR_INSUFFICIENT_BUFFER, FILETIME, HANDLE, HLOCAL, HMODULE, LocalFree, MAX_PATH,
     STATUS_BUFFER_OVERFLOW, STATUS_BUFFER_TOO_SMALL, STATUS_INFO_LENGTH_MISMATCH, UNICODE_STRING,
 };
-use windows::Win32::Security::{GetTokenInformation, TokenUser, TOKEN_QUERY, TOKEN_USER};
+use windows::Win32::Security::{GetTokenInformation, TOKEN_QUERY, TOKEN_USER, TokenUser};
 use windows::Win32::System::Diagnostics::Debug::ReadProcessMemory;
+use windows::Win32::System::Diagnostics::ToolHelp::PROCESSENTRY32W;
 use windows::Win32::System::Memory::{
-    GetProcessHeap, HeapAlloc, HeapFree, VirtualQueryEx, HEAP_ZERO_MEMORY, MEMORY_BASIC_INFORMATION,
+    GetProcessHeap, HEAP_ZERO_MEMORY, HeapAlloc, HeapFree, MEMORY_BASIC_INFORMATION, VirtualQueryEx,
+};
+use windows::Win32::System::ProcessStatus::{
+    GetModuleFileNameExW, GetProcessMemoryInfo, PROCESS_MEMORY_COUNTERS_EX,
 };
-use windows::Win32::System::ProcessStatus::GetModuleFileNameExW;
 use windows::Win32::System::RemoteDesktop::ProcessIdToSessionId;
 use windows::Win32::System::SystemInformation::OSVERSIONINFOEXW;
 use windows::Win32::System::Threading::{
-    GetProcessIoCounters, GetProcessTimes, GetSystemTimes, OpenProcess, OpenProcessToken,
-    CREATE_NO_WINDOW, IO_COUNTERS, PEB, PROCESS_BASIC_INFORMATION, PROCESS_QUERY_INFORMATION,
-    PROCESS_QUERY_LIMITED_INFORMATION, PROCESS_VM_READ,
+    CREATE_NO_WINDOW, GetExitCodeProcess, GetProcessHandleCount, GetProcessIoCounters,
+    GetProcessTimes, GetSystemTimes, IO_COUNTERS, OpenProcess, OpenProcessToken, PEB,
+    PROCESS_BASIC_INFORMATION, PROCESS_QUERY_INFORMATION, PROCESS_QUERY_LIMITED_INFORMATION,
+    PROCESS_VM_READ,
 };
 use windows::Win32::UI::Shell::CommandLineToArgvW;
+use windows::core::PCWSTR;
+
+use super::MINIMUM_CPU_UPDATE_INTERVAL;
+
+const FILETIMES_PER_MILLISECONDS: u64 = 10_000; // 100 nanosecond units
 
 impl fmt::Display for ProcessStatus {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -87,12 +95,14 @@ unsafe fn get_process_user_id(process: &mut ProcessInner, refresh_kind: ProcessR
 
     impl<T> HeapWrap<T> {
         unsafe fn new(size: u32) -> Option<Self> {
-            let ptr = HeapAlloc(GetProcessHeap().ok()?, HEAP_ZERO_MEMORY, size as _) as *mut T;
-            if ptr.is_null() {
-                sysinfo_debug!("HeapAlloc failed");
-                None
-            } else {
-                Some(Self(ptr))
+            unsafe {
+                let ptr = HeapAlloc(GetProcessHeap().ok()?, HEAP_ZERO_MEMORY, size as _) as *mut T;
+                if ptr.is_null() {
+                    sysinfo_debug!("HeapAlloc failed");
+                    None
+                } else {
+                    Some(Self(ptr))
+                }
             }
         }
     }
@@ -124,41 +134,45 @@ unsafe fn get_process_user_id(process: &mut ProcessInner, refresh_kind: ProcessR
 
     let mut token = Default::default();
 
-    if OpenProcessToken(handle, TOKEN_QUERY, &mut token).is_err() {
-        sysinfo_debug!("OpenProcessToken failed");
-        return;
-    }
+    unsafe {
+        if OpenProcessToken(handle, TOKEN_QUERY, &mut token).is_err() {
+            sysinfo_debug!("OpenProcessToken failed");
+            return;
+        }
 
-    let token = match HandleWrapper::new(token) {
-        Some(token) => token,
-        None => return,
-    };
+        let token = match HandleWrapper::new(token) {
+            Some(token) => token,
+            None => return,
+        };
 
-    let mut size = 0;
+        let mut size = 0;
 
-    if let Err(err) = GetTokenInformation(*token, TokenUser, None, 0, &mut size) {
-        if err.code() != ERROR_INSUFFICIENT_BUFFER.to_hresult() {
+        if let Err(err) = GetTokenInformation(*token, TokenUser, None, 0, &mut size)
+            && err.code() != ERROR_INSUFFICIENT_BUFFER.to_hresult()
+        {
             sysinfo_debug!("GetTokenInformation failed, error: {:?}", err);
             return;
         }
-    }
 
-    let ptu: HeapWrap<TOKEN_USER> = match HeapWrap::new(size) {
-        Some(ptu) => ptu,
-        None => return,
-    };
+        let ptu: HeapWrap<TOKEN_USER> = match HeapWrap::new(size) {
+            Some(ptu) => ptu,
+            None => return,
+        };
 
-    if let Err(_err) = GetTokenInformation(*token, TokenUser, Some(ptu.0.cast()), size, &mut size) {
-        sysinfo_debug!(
-            "GetTokenInformation failed (returned {_err:?}), error: {:?}",
-            io::Error::last_os_error()
-        );
-        return;
-    }
+        if let Err(_err) =
+            GetTokenInformation(*token, TokenUser, Some(ptu.0.cast()), size, &mut size)
+        {
+            sysinfo_debug!(
+                "GetTokenInformation failed (returned {_err:?}), error: {:?}",
+                io::Error::last_os_error()
+            );
+            return;
+        }
 
-    // We force this check to prevent overwritting a `Some` with a `None`.
-    if let Some(uid) = Sid::from_psid((*ptu.0).User.Sid).map(Uid) {
-        process.user_id = Some(uid);
+        // We force this check to prevent overwriting a `Some` with a `None`.
+        if let Some(uid) = Sid::from_psid((*ptu.0).User.Sid).map(Uid) {
+            process.user_id = Some(uid);
+        }
     }
 }
 
@@ -189,6 +203,8 @@ pub(crate) struct ProcessInner {
     old_written_bytes: u64,
     read_bytes: u64,
     written_bytes: u64,
+    accumulated_cpu_time: u64,
+    exists: bool,
 }
 
 struct CPUsageCalculationValues {
@@ -196,6 +212,7 @@ struct CPUsageCalculationValues {
     old_process_user_cpu: u64,
     old_system_sys_cpu: u64,
     old_system_user_cpu: u64,
+    last_update: Instant,
 }
 
 impl CPUsageCalculationValues {
@@ -205,6 +222,7 @@ impl CPUsageCalculationValues {
             old_process_user_cpu: 0,
             old_system_sys_cpu: 0,
             old_system_user_cpu: 0,
+            last_update: Instant::now(),
         }
     }
 }
@@ -225,26 +243,17 @@ fn windows_8_1_or_newer() -> &'static bool {
     })
 }
 
-#[cfg(feature = "debug")]
-unsafe fn display_ntstatus_error(ntstatus: windows::core::HRESULT) {
-    let code = ntstatus.0;
-    let message = ntstatus.message();
-    sysinfo_debug!(
-        "Couldn't get process infos: NtQuerySystemInformation returned {}: {}",
-        code,
-        message
-    );
-}
-
 unsafe fn get_exe(process_handler: &HandleWrapper) -> Option<PathBuf> {
     let mut exe_buf = [0u16; MAX_PATH as usize + 1];
-    GetModuleFileNameExW(
-        **process_handler,
-        HINSTANCE::default(),
-        exe_buf.as_mut_slice(),
-    );
+    unsafe {
+        GetModuleFileNameExW(
+            Some(**process_handler),
+            Some(HMODULE::default()),
+            exe_buf.as_mut_slice(),
+        );
 
-    Some(PathBuf::from(null_terminated_wchar_to_string(&exe_buf)))
+        Some(PathBuf::from(null_terminated_wchar_to_string(&exe_buf)))
+    }
 }
 
 impl ProcessInner {
@@ -278,6 +287,8 @@ impl ProcessInner {
             old_written_bytes: 0,
             read_bytes: 0,
             written_bytes: 0,
+            accumulated_cpu_time: 0,
+            exists: true,
         }
     }
 
@@ -287,7 +298,6 @@ impl ProcessInner {
         nb_cpus: u64,
         now: u64,
         refresh_parent: bool,
-        pi: &SYSTEM_PROCESS_INFORMATION,
     ) {
         if refresh_kind.cpu() {
             compute_cpu_usage(self, nb_cpus);
@@ -296,8 +306,21 @@ impl ProcessInner {
             update_disk_usage(self);
         }
         if refresh_kind.memory() {
-            self.memory = pi.WorkingSetSize as _;
-            self.virtual_memory = pi.VirtualSize as _;
+            let mut mem_info = PROCESS_MEMORY_COUNTERS_EX::default();
+            if let Some(handle) = self.get_handle() {
+                if let Err(_error) = unsafe {
+                    GetProcessMemoryInfo(
+                        handle,
+                        &mut mem_info as *mut _ as *mut _,
+                        std::mem::size_of_val::<PROCESS_MEMORY_COUNTERS_EX>(&mem_info) as _,
+                    )
+                } {
+                    sysinfo_debug!("GetProcessMemoryInfo failed: {_error:?}");
+                } else {
+                    self.memory = mem_info.WorkingSetSize as _;
+                    self.virtual_memory = mem_info.PrivateUsage as _;
+                }
+            }
         }
         unsafe {
             get_process_user_id(self, refresh_kind);
@@ -315,12 +338,28 @@ impl ProcessInner {
         self.updated = true;
     }
 
-    pub(crate) fn get_handle(&self) -> Option<HANDLE> {
-        self.handle.as_ref().map(|h| ***h)
+    pub(crate) fn from_process_entry(entry: &PROCESSENTRY32W, now: u64) -> Self {
+        let pid = Pid::from_u32(entry.th32ProcessID);
+        let name = match OsString::from_str(
+            String::from_utf16_lossy(&entry.szExeFile).trim_end_matches('\0'),
+        ) {
+            Ok(name) => name,
+            Err(_) => format!("<no name> Process {pid}").into(),
+        };
+        let ppid = {
+            if entry.th32ParentProcessID == 0 {
+                // no parent pid
+                None
+            } else {
+                Some(Pid::from_u32(entry.th32ParentProcessID))
+            }
+        };
+
+        Self::new(pid, ppid, now, name)
     }
 
-    pub(crate) fn get_start_time(&self) -> Option<u64> {
-        self.handle.as_ref().map(|handle| get_start_time(***handle))
+    pub(crate) fn get_handle(&self) -> Option<HANDLE> {
+        self.handle.as_ref().map(|h| ***h)
     }
 
     pub(crate) fn kill_with(&self, signal: Signal) -> Option<bool> {
@@ -390,6 +429,10 @@ impl ProcessInner {
         self.cpu_usage
     }
 
+    pub(crate) fn accumulated_cpu_time(&self) -> u64 {
+        self.accumulated_cpu_time
+    }
+
     pub(crate) fn disk_usage(&self) -> DiskUsage {
         DiskUsage {
             written_bytes: self.written_bytes.saturating_sub(self.old_written_bytes),
@@ -415,18 +458,30 @@ impl ProcessInner {
         None
     }
 
-    pub(crate) fn wait(&self) {
+    pub(crate) fn wait(&self) -> Option<ExitStatus> {
         if let Some(handle) = self.get_handle() {
             while is_proc_running(handle) {
                 if get_start_time(handle) != self.start_time() {
                     // PID owner changed so the previous process was finished!
-                    return;
+                    sysinfo_debug!("PID owner changed so cannot get old process exit status");
+                    return None;
                 }
                 std::thread::sleep(std::time::Duration::from_millis(10));
             }
+            let mut exit_status = 0;
+            unsafe {
+                match GetExitCodeProcess(handle, &mut exit_status) {
+                    Ok(_) => Some(ExitStatus::from_raw(exit_status)),
+                    Err(_error) => {
+                        sysinfo_debug!("failed to retrieve process exit status: {_error:?}");
+                        None
+                    }
+                }
+            }
         } else {
             // In this case, we can't do anything so we just return.
             sysinfo_debug!("can't wait on this process so returning");
+            None
         }
     }
 
@@ -445,23 +500,53 @@ impl ProcessInner {
     }
 
     pub(crate) fn switch_updated(&mut self) -> bool {
-         std::mem::replace(&mut self.updated, false)
+        std::mem::replace(&mut self.updated, false)
+    }
+
+    pub(crate) fn set_nonexistent(&mut self) {
+        self.exists = false;
+    }
+
+    pub(crate) fn exists(&self) -> bool {
+        self.exists
+    }
+
+    pub(crate) fn open_files(&self) -> Option<usize> {
+        if let Some(ref handle) = self.handle {
+            let mut handles_count = 0;
+            unsafe {
+                if let Err(_error) = GetProcessHandleCount(***handle, &mut handles_count) {
+                    sysinfo_debug!("GetProcessHandleCount failed: {_error:?}");
+                    None
+                } else {
+                    Some(handles_count as _)
+                }
+            }
+        } else {
+            None
+        }
+    }
+
+    pub(crate) fn open_files_limit(&self) -> Option<usize> {
+        crate::System::open_files_limit()
     }
 }
 
 #[inline]
 unsafe fn get_process_times(handle: HANDLE) -> u64 {
-    let mut fstart: FILETIME = zeroed();
-    let mut x = zeroed();
+    unsafe {
+        let mut fstart: FILETIME = zeroed();
+        let mut x = zeroed();
 
-    let _err = GetProcessTimes(
-        handle,
-        &mut fstart as *mut FILETIME,
-        &mut x as *mut FILETIME,
-        &mut x as *mut FILETIME,
-        &mut x as *mut FILETIME,
-    );
-    filetime_to_u64(fstart)
+        let _err = GetProcessTimes(
+            handle,
+            &mut fstart as *mut FILETIME,
+            &mut x as *mut FILETIME,
+            &mut x as *mut FILETIME,
+            &mut x as *mut FILETIME,
+        );
+        filetime_to_u64(fstart)
+    }
 }
 
 // On Windows, the root folder is always the current drive. So we get it from its `cwd`.
@@ -512,78 +597,82 @@ unsafe fn ph_query_process_variable_size(
 ) -> Option<Vec<u16>> {
     let mut return_length = MaybeUninit::<u32>::uninit();
 
-    if let Err(err) = NtQueryInformationProcess(
-        process_handle,
-        process_information_class as _,
-        null_mut(),
-        0,
-        return_length.as_mut_ptr() as *mut _,
-    )
-    .ok()
-    {
-        if ![
-            STATUS_BUFFER_OVERFLOW.into(),
-            STATUS_BUFFER_TOO_SMALL.into(),
-            STATUS_INFO_LENGTH_MISMATCH.into(),
-        ]
-        .contains(&err.code())
+    unsafe {
+        if let Err(err) = NtQueryInformationProcess(
+            process_handle,
+            process_information_class as _,
+            null_mut(),
+            0,
+            return_length.as_mut_ptr() as *mut _,
+        )
+        .ok()
+            && ![
+                STATUS_BUFFER_OVERFLOW.into(),
+                STATUS_BUFFER_TOO_SMALL.into(),
+                STATUS_INFO_LENGTH_MISMATCH.into(),
+            ]
+            .contains(&err.code())
         {
             return None;
         }
-    }
 
-    let mut return_length = return_length.assume_init();
-    let buf_len = (return_length as usize) / 2;
-    let mut buffer: Vec<u16> = Vec::with_capacity(buf_len + 1);
-    if NtQueryInformationProcess(
-        process_handle,
-        process_information_class as _,
-        buffer.as_mut_ptr() as *mut _,
-        return_length,
-        &mut return_length as *mut _,
-    )
-    .is_err()
-    {
-        return None;
+        let mut return_length = return_length.assume_init();
+        let buf_len = (return_length as usize) / 2;
+        let mut buffer: Vec<u16> = Vec::with_capacity(buf_len + 1);
+        if NtQueryInformationProcess(
+            process_handle,
+            process_information_class as _,
+            buffer.as_mut_ptr() as *mut _,
+            return_length,
+            &mut return_length as *mut _,
+        )
+        .is_err()
+        {
+            return None;
+        }
+        buffer.set_len(buf_len);
+        buffer.push(0);
+        Some(buffer)
     }
-    buffer.set_len(buf_len);
-    buffer.push(0);
-    Some(buffer)
 }
 
 unsafe fn get_cmdline_from_buffer(buffer: PCWSTR) -> Vec<OsString> {
     // Get argc and argv from the command line
     let mut argc = MaybeUninit::<i32>::uninit();
-    let argv_p = CommandLineToArgvW(buffer, argc.as_mut_ptr());
-    if argv_p.is_null() {
-        return Vec::new();
+    unsafe {
+        let argv_p = CommandLineToArgvW(buffer, argc.as_mut_ptr());
+        if argv_p.is_null() {
+            return Vec::new();
+        }
+        let argc = argc.assume_init();
+        let argv = std::slice::from_raw_parts(argv_p, argc as usize);
+
+        let mut res = Vec::new();
+        for arg in argv {
+            res.push(OsString::from_wide(arg.as_wide()));
+        }
+
+        let _err = LocalFree(Some(HLOCAL(argv_p as _)));
+
+        res
     }
-    let argc = argc.assume_init();
-    let argv = std::slice::from_raw_parts(argv_p, argc as usize);
-
-    let mut res = Vec::new();
-    for arg in argv {
-        res.push(OsString::from_wide(arg.as_wide()));
-    }
-
-    let _err = LocalFree(HLOCAL(argv_p as _));
-
-    res
 }
 
 unsafe fn get_region_size(handle: HANDLE, ptr: *const c_void) -> Result<usize, &'static str> {
     let mut meminfo = MaybeUninit::<MEMORY_BASIC_INFORMATION>::uninit();
-    if VirtualQueryEx(
-        handle,
-        Some(ptr),
-        meminfo.as_mut_ptr().cast(),
-        size_of::<MEMORY_BASIC_INFORMATION>(),
-    ) == 0
-    {
-        return Err("Unable to read process memory information");
+    unsafe {
+        if VirtualQueryEx(
+            handle,
+            Some(ptr),
+            meminfo.as_mut_ptr().cast(),
+            size_of::<MEMORY_BASIC_INFORMATION>(),
+        ) == 0
+        {
+            return Err("Unable to read process memory information");
+        }
+        let meminfo = meminfo.assume_init();
+        Ok((meminfo.RegionSize as isize - ptr.offset_from(meminfo.BaseAddress)) as usize)
     }
-    let meminfo = meminfo.assume_init();
-    Ok((meminfo.RegionSize as isize - ptr.offset_from(meminfo.BaseAddress)) as usize)
 }
 
 unsafe fn get_process_data(
@@ -594,25 +683,27 @@ unsafe fn get_process_data(
     let mut buffer: Vec<u16> = Vec::with_capacity(size / 2 + 1);
     let mut bytes_read = 0;
 
-    if ReadProcessMemory(
-        handle,
-        ptr,
-        buffer.as_mut_ptr().cast(),
-        size,
-        Some(&mut bytes_read),
-    )
-    .is_err()
-    {
-        return Err("Unable to read process data");
-    }
+    unsafe {
+        if ReadProcessMemory(
+            handle,
+            ptr,
+            buffer.as_mut_ptr().cast(),
+            size,
+            Some(&mut bytes_read),
+        )
+        .is_err()
+        {
+            return Err("Unable to read process data");
+        }
 
-    // Documentation states that the function fails if not all data is accessible.
-    if bytes_read != size {
-        return Err("ReadProcessMemory returned unexpected number of bytes read");
-    }
+        // Documentation states that the function fails if not all data is accessible.
+        if bytes_read != size {
+            return Err("ReadProcessMemory returned unexpected number of bytes read");
+        }
 
-    buffer.set_len(size / 2);
-    buffer.push(0);
+        buffer.set_len(size / 2);
+        buffer.push(0);
+    }
 
     Ok(buffer)
 }
@@ -676,87 +767,134 @@ unsafe fn get_process_params(
 
     // First check if target process is running in wow64 compatibility emulator
     let mut pwow32info = MaybeUninit::<*const c_void>::uninit();
-    if NtQueryInformationProcess(
-        handle,
-        ProcessWow64Information,
-        pwow32info.as_mut_ptr().cast(),
-        size_of::<*const c_void>() as u32,
-        null_mut(),
-    )
-    .is_err()
-    {
-        sysinfo_debug!("Unable to check WOW64 information about the process");
-        return;
-    }
-    let pwow32info = pwow32info.assume_init();
-
-    // Get parent and PEB64 from PROCESS_BASIC_INFORMATION
-    // PEB64 will only be used if the target is 64 bit process
-    let pinfo = if refresh_parent || pwow32info.is_null() {
-        let mut pbasicinfo = MaybeUninit::<PROCESS_BASIC_INFORMATION>::uninit();
+    unsafe {
         if NtQueryInformationProcess(
             handle,
-            ProcessBasicInformation,
-            pbasicinfo.as_mut_ptr().cast(),
-            size_of::<PROCESS_BASIC_INFORMATION>() as u32,
+            ProcessWow64Information,
+            pwow32info.as_mut_ptr().cast(),
+            size_of::<*const c_void>() as u32,
             null_mut(),
         )
         .is_err()
         {
-            sysinfo_debug!("Unable to get basic process information");
+            sysinfo_debug!("Unable to check WOW64 information about the process");
             return;
         }
-        let pinfo = pbasicinfo.assume_init();
+        let pwow32info = pwow32info.assume_init();
 
-        let ppid: usize = pinfo.InheritedFromUniqueProcessId as _;
-        let parent = if ppid != 0 {
-            Some(Pid(pinfo.InheritedFromUniqueProcessId as _))
+        // Get parent and PEB64 from PROCESS_BASIC_INFORMATION
+        // PEB64 will only be used if the target is 64 bit process
+        let pinfo = if refresh_parent || pwow32info.is_null() {
+            let mut pbasicinfo = MaybeUninit::<PROCESS_BASIC_INFORMATION>::uninit();
+            if NtQueryInformationProcess(
+                handle,
+                ProcessBasicInformation,
+                pbasicinfo.as_mut_ptr().cast(),
+                size_of::<PROCESS_BASIC_INFORMATION>() as u32,
+                null_mut(),
+            )
+            .is_err()
+            {
+                sysinfo_debug!("Unable to get basic process information");
+                return;
+            }
+            let pinfo = pbasicinfo.assume_init();
+
+            let ppid: usize = pinfo.InheritedFromUniqueProcessId as _;
+            let parent = if ppid != 0 {
+                Some(Pid(pinfo.InheritedFromUniqueProcessId as _))
+            } else {
+                None
+            };
+            process.parent = parent;
+
+            if !has_anything_to_update {
+                return;
+            }
+
+            Some(pinfo)
         } else {
             None
         };
-        process.parent = parent;
+
+        if pwow32info.is_null() {
+            // target is a 64 bit process
+
+            let Some(pinfo) = pinfo else { return };
+
+            let mut peb = MaybeUninit::<PEB>::uninit();
+            if ReadProcessMemory(
+                handle,
+                pinfo.PebBaseAddress.cast(),
+                peb.as_mut_ptr().cast(),
+                size_of::<PEB>(),
+                None,
+            )
+            .is_err()
+            {
+                sysinfo_debug!("Unable to read process PEB");
+                return;
+            }
+
+            let peb = peb.assume_init();
+
+            let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS>::uninit();
+            if ReadProcessMemory(
+                handle,
+                peb.ProcessParameters.cast(),
+                proc_params.as_mut_ptr().cast(),
+                size_of::<RTL_USER_PROCESS_PARAMETERS>(),
+                None,
+            )
+            .is_err()
+            {
+                sysinfo_debug!("Unable to read process parameters");
+                return;
+            }
+
+            let proc_params = proc_params.assume_init();
+            get_cmd_line(&proc_params, handle, refresh_kind, &mut process.cmd);
+            get_proc_env(&proc_params, handle, refresh_kind, &mut process.environ);
+            get_cwd_and_root(
+                &proc_params,
+                handle,
+                refresh_kind,
+                &mut process.cwd,
+                &mut process.root,
+            );
+        }
+        // target is a 32 bit process in wow64 mode
 
         if !has_anything_to_update {
             return;
         }
 
-        Some(pinfo)
-    } else {
-        None
-    };
-
-    if pwow32info.is_null() {
-        // target is a 64 bit process
-
-        let Some(pinfo) = pinfo else { return };
-
-        let mut peb = MaybeUninit::<PEB>::uninit();
+        let mut peb32 = MaybeUninit::<PEB32>::uninit();
         if ReadProcessMemory(
             handle,
-            pinfo.PebBaseAddress.cast(),
-            peb.as_mut_ptr().cast(),
-            size_of::<PEB>(),
+            pwow32info,
+            peb32.as_mut_ptr().cast(),
+            size_of::<PEB32>(),
             None,
         )
         .is_err()
         {
-            sysinfo_debug!("Unable to read process PEB");
+            sysinfo_debug!("Unable to read PEB32");
             return;
         }
+        let peb32 = peb32.assume_init();
 
-        let peb = peb.assume_init();
-
-        let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS>::uninit();
+        let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS32>::uninit();
         if ReadProcessMemory(
             handle,
-            peb.ProcessParameters.cast(),
+            peb32.ProcessParameters as *mut _,
             proc_params.as_mut_ptr().cast(),
-            size_of::<RTL_USER_PROCESS_PARAMETERS>(),
+            size_of::<RTL_USER_PROCESS_PARAMETERS32>(),
             None,
         )
         .is_err()
         {
-            sysinfo_debug!("Unable to read process parameters");
+            sysinfo_debug!("Unable to read 32 bit process parameters");
             return;
         }
 
@@ -771,50 +909,6 @@ unsafe fn get_process_params(
             &mut process.root,
         );
     }
-    // target is a 32 bit process in wow64 mode
-
-    if !has_anything_to_update {
-        return;
-    }
-
-    let mut peb32 = MaybeUninit::<PEB32>::uninit();
-    if ReadProcessMemory(
-        handle,
-        pwow32info,
-        peb32.as_mut_ptr().cast(),
-        size_of::<PEB32>(),
-        None,
-    )
-    .is_err()
-    {
-        sysinfo_debug!("Unable to read PEB32");
-        return;
-    }
-    let peb32 = peb32.assume_init();
-
-    let mut proc_params = MaybeUninit::<RTL_USER_PROCESS_PARAMETERS32>::uninit();
-    if ReadProcessMemory(
-        handle,
-        peb32.ProcessParameters as *mut _,
-        proc_params.as_mut_ptr().cast(),
-        size_of::<RTL_USER_PROCESS_PARAMETERS32>(),
-        None,
-    )
-    .is_err()
-    {
-        sysinfo_debug!("Unable to read 32 bit process parameters");
-        return;
-    }
-    let proc_params = proc_params.assume_init();
-    get_cmd_line(&proc_params, handle, refresh_kind, &mut process.cmd);
-    get_proc_env(&proc_params, handle, refresh_kind, &mut process.environ);
-    get_cwd_and_root(
-        &proc_params,
-        handle,
-        refresh_kind,
-        &mut process.cwd,
-        &mut process.root,
-    );
 }
 
 fn get_cwd_and_root<T: RtlUserProcessParameters>(
@@ -903,13 +997,16 @@ fn get_proc_env<T: RtlUserProcessParameters>(
     }
     match params.get_environ(handle) {
         Ok(buffer) => {
-            let equals = "=".encode_utf16().next().unwrap();
+            let Some(equals) = "=".encode_utf16().next() else {
+                sysinfo_debug!("not found `=` while trying to parse proc variable");
+                return;
+            };
             let raw_env = buffer;
             environ.clear();
             let mut begin = 0;
             while let Some(offset) = raw_env[begin..].iter().position(|&c| c == 0) {
                 let end = begin + offset;
-                if raw_env[begin..end].iter().any(|&c| c == equals) {
+                if raw_env[begin..end].contains(&equals) {
                     environ.push(OsString::from_wide(&raw_env[begin..end]));
                     begin = end + 1;
                 } else {
@@ -940,16 +1037,14 @@ pub(crate) fn get_executable_path(_pid: Pid) -> Option<PathBuf> {
 
 #[inline]
 fn check_sub(a: u64, b: u64) -> u64 {
-    if a < b {
-        a
-    } else {
-        a - b
-    }
+    if a < b { a } else { a - b }
 }
 
 /// Before changing this function, you must consider the following:
 /// <https://github.com/GuillaumeGomez/sysinfo/issues/459>
 pub(crate) fn compute_cpu_usage(p: &mut ProcessInner, nb_cpus: u64) {
+    let need_update = p.cpu_calc_values.last_update.elapsed() >= MINIMUM_CPU_UPDATE_INTERVAL;
+
     unsafe {
         let mut ftime: FILETIME = zeroed();
         let mut fsys: FILETIME = zeroed();
@@ -961,19 +1056,26 @@ pub(crate) fn compute_cpu_usage(p: &mut ProcessInner, nb_cpus: u64) {
         if let Some(handle) = p.get_handle() {
             let _err = GetProcessTimes(handle, &mut ftime, &mut ftime, &mut fsys, &mut fuser);
         }
-        // FIXME: should these values be stored in one place to make use of
-        // `MINIMUM_CPU_UPDATE_INTERVAL`?
+
+        // system times have changed, we need to get most recent system times
+        // and update the cpu times cache, as well as global_kernel_time and global_user_time
         let _err = GetSystemTimes(
             Some(&mut fglobal_idle_time),
             Some(&mut fglobal_kernel_time),
             Some(&mut fglobal_user_time),
         );
 
+        p.cpu_calc_values.last_update = Instant::now();
+
         let sys = filetime_to_u64(fsys);
         let user = filetime_to_u64(fuser);
         let global_kernel_time = filetime_to_u64(fglobal_kernel_time);
         let global_user_time = filetime_to_u64(fglobal_user_time);
 
+        p.accumulated_cpu_time = user.saturating_add(sys) / FILETIMES_PER_MILLISECONDS;
+        if !need_update {
+            return;
+        }
         let delta_global_kernel_time =
             check_sub(global_kernel_time, p.cpu_calc_values.old_system_sys_cpu);
         let delta_global_user_time =
@@ -1019,5 +1121,5 @@ pub(crate) fn update_disk_usage(p: &mut ProcessInner) {
 
 #[inline(always)]
 const fn filetime_to_u64(ft: FILETIME) -> u64 {
-    (ft.dwHighDateTime as u64) << 32 | (ft.dwLowDateTime as u64)
+    ((ft.dwHighDateTime as u64) << 32) | (ft.dwLowDateTime as u64)
 }
