@@ -1,28 +1,25 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use crate::{Cpu, CpuRefreshKind, LoadAvg, MemoryRefreshKind, Pid, ProcessesToUpdate, ProcessRefreshKind};
+use crate::{
+    Cpu, CpuRefreshKind, LoadAvg, MemoryRefreshKind, Pid, ProcessRefreshKind, ProcessesToUpdate,
+};
 
 use crate::sys::cpu::*;
 use crate::{Process, ProcessInner};
 
-use crate::utils::into_iter;
-
-use std::cell::UnsafeCell;
 use std::collections::HashMap;
-use std::ffi::{OsStr, OsString};
+use std::ffi::OsStr;
 use std::mem::{size_of, zeroed};
-use std::os::windows::ffi::{OsStrExt, OsStringExt};
-use std::ptr;
-use std::sync::atomic::{AtomicUsize, Ordering};
+use std::os::windows::ffi::OsStrExt;
 use std::time::{Duration, SystemTime};
 
-use ntapi::ntexapi::SYSTEM_PROCESS_INFORMATION;
-use windows::core::{PCWSTR, PWSTR};
-use windows::Wdk::System::SystemInformation::{NtQuerySystemInformation, SystemProcessInformation};
-use windows::Win32::Foundation::{self, HANDLE, STATUS_INFO_LENGTH_MISMATCH, STILL_ACTIVE};
+use windows::Win32::Foundation::{self, HANDLE, STILL_ACTIVE};
+use windows::Win32::System::Diagnostics::ToolHelp::{
+    CreateToolhelp32Snapshot, PROCESSENTRY32W, Process32FirstW, Process32NextW, TH32CS_SNAPPROCESS,
+};
 use windows::Win32::System::ProcessStatus::{K32GetPerformanceInfo, PERFORMANCE_INFORMATION};
 use windows::Win32::System::Registry::{
-    RegCloseKey, RegOpenKeyExW, RegQueryValueExW, HKEY, HKEY_LOCAL_MACHINE, KEY_READ, REG_NONE,
+    HKEY, HKEY_LOCAL_MACHINE, KEY_READ, REG_NONE, RegCloseKey, RegOpenKeyExW, RegQueryValueExW,
 };
 use windows::Win32::System::SystemInformation::{self, GetSystemInfo};
 use windows::Win32::System::SystemInformation::{
@@ -30,6 +27,7 @@ use windows::Win32::System::SystemInformation::{
     MEMORYSTATUSEX, SYSTEM_INFO,
 };
 use windows::Win32::System::Threading::GetExitCodeProcess;
+use windows::core::{Owned, PCWSTR, PWSTR};
 
 declare_signals! {
     (),
@@ -54,16 +52,20 @@ impl SystemInner {
     }
 }
 
-// Useful for parallel iterations.
-struct Wrap<T>(T);
-
-#[allow(clippy::non_send_fields_in_send_ty)]
-unsafe impl<T> Send for Wrap<T> {}
-unsafe impl<T> Sync for Wrap<T> {}
-
+/// Calculates system boot time in seconds with improved precision.
+/// Uses nanoseconds throughout to avoid rounding errors in uptime calculation,
+/// converting to seconds only at the end for stable results. Result is capped
+/// within u64 limits to handle edge cases.
 unsafe fn boot_time() -> u64 {
     match SystemTime::now().duration_since(SystemTime::UNIX_EPOCH) {
-        Ok(n) => n.as_secs().saturating_sub(GetTickCount64() / 1_000),
+        Ok(n) => {
+            let system_time_ns = n.as_nanos();
+            // milliseconds to nanoseconds
+            let tick_count_ns = unsafe { GetTickCount64() } as u128 * 1_000_000;
+            // nanoseconds to seconds
+            let boot_time_sec = system_time_ns.saturating_sub(tick_count_ns) / 1_000_000_000;
+            boot_time_sec.try_into().unwrap_or(u64::MAX)
+        }
         Err(_e) => {
             sysinfo_debug!("Failed to compute boot time: {:?}", _e);
             0
@@ -94,50 +96,53 @@ impl SystemInner {
         }
     }
 
+    fn initialize_cpu_counters(&mut self, refresh_kind: CpuRefreshKind) {
+        if let Some(ref mut query) = self.query {
+            add_english_counter(
+                r"\Processor(_Total)\% Idle Time".to_string(),
+                query,
+                &mut self.cpus.global.key_used,
+                "tot_0".to_owned(),
+            );
+            for (pos, proc_) in self.cpus.iter_mut(refresh_kind).enumerate() {
+                add_english_counter(
+                    format!(r"\Processor({pos})\% Idle Time"),
+                    query,
+                    get_key_used(proc_),
+                    format!("{pos}_0"),
+                );
+            }
+        }
+    }
+
     pub(crate) fn refresh_cpu_specifics(&mut self, refresh_kind: CpuRefreshKind) {
         if self.query.is_none() {
-            self.query = Query::new();
-            if let Some(ref mut query) = self.query {
-                add_english_counter(
-                    r"\Processor(_Total)\% Idle Time".to_string(),
-                    query,
-                    &mut self.cpus.global.key_used,
-                    "tot_0".to_owned(),
-                );
-                for (pos, proc_) in self.cpus.iter_mut(refresh_kind).enumerate() {
-                    add_english_counter(
-                        format!(r"\Processor({pos})\% Idle Time"),
-                        query,
-                        get_key_used(proc_),
-                        format!("{pos}_0"),
-                    );
-                }
-            }
+            self.query = Query::new(false);
+            self.initialize_cpu_counters(refresh_kind);
+        } else if self.cpus.global.key_used.is_none() {
+            self.query = Query::new(true);
+            self.initialize_cpu_counters(refresh_kind);
         }
         if let Some(ref mut query) = self.query {
             query.refresh();
-            let mut total_idle_time = None;
             if let Some(ref key_used) = self.cpus.global.key_used {
-                total_idle_time = Some(
-                    query
-                        .get(&key_used.unique_id)
-                        .expect("global_key_idle disappeared"),
-                );
-            }
-            if let Some(total_idle_time) = total_idle_time {
-                self.cpus.global.set_cpu_usage(100.0 - total_idle_time);
+                #[allow(clippy::single_match)]
+                match query.get(&key_used.unique_id) {
+                    Some(total_idle_time) => {
+                        self.cpus.global.set_cpu_usage(100.0 - total_idle_time);
+                    }
+                    None => sysinfo_debug!("global_key_idle disappeared"),
+                }
             }
             for cpu in self.cpus.iter_mut(refresh_kind) {
-                let mut idle_time = None;
                 if let Some(ref key_used) = *get_key_used(cpu) {
-                    idle_time = Some(
-                        query
-                            .get(&key_used.unique_id)
-                            .expect("key_used disappeared"),
-                    );
-                }
-                if let Some(idle_time) = idle_time {
-                    cpu.inner.set_cpu_usage(100.0 - idle_time);
+                    #[allow(clippy::single_match)]
+                    match query.get(&key_used.unique_id) {
+                        Some(idle_time) => {
+                            cpu.inner.set_cpu_usage(100.0 - idle_time);
+                        }
+                        None => sysinfo_debug!("key_used disappeared"),
+                    }
                 }
             }
             if refresh_kind.frequency() {
@@ -212,132 +217,86 @@ impl SystemInner {
             }
         };
 
-        // Windows 10 notebook requires at least 512KiB of memory to make it in one go
-        let mut buffer_size = 512 * 1024;
-        let mut process_information: Vec<u8> = Vec::with_capacity(buffer_size);
+        let now = get_now();
 
+        let nb_cpus = if refresh_kind.cpu() {
+            self.cpus.len() as u64
+        } else {
+            0
+        };
+
+        // Use the amazing and cool CreateToolhelp32Snapshot function.
+        // Take a snapshot of all running processes. Match the result to an error
+        let snapshot = match unsafe { CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0) } {
+            Ok(handle) => handle,
+            Err(_err) => {
+                sysinfo_debug!(
+                    "Error capturing process snapshot: CreateToolhelp32Snapshot returned {}",
+                    _err
+                );
+                return 0;
+            }
+        };
+
+        // This owns the above handle and makes sure that close will be called when dropped.
+        let snapshot = unsafe { Owned::new(snapshot) };
+
+        // https://learn.microsoft.com/en-us/windows/win32/api/tlhelp32/ns-tlhelp32-processentry32w
+        // Microsoft documentation states that for PROCESSENTRY32W, before calling Process32FirstW,
+        // the 'dwSize' field MUST be set to the size of the PROCESSENTRY32W. Otherwise, Process32FirstW fails.
+        let mut process_entry = PROCESSENTRY32W {
+            dwSize: size_of::<PROCESSENTRY32W>() as u32,
+            ..Default::default()
+        };
+
+        let mut num_procs = 0; // keep track of the number of updated processes
+        let process_list = &mut self.process_list;
+
+        // process the first process
         unsafe {
-            loop {
-                let mut cb_needed = 0;
-                // reserve(n) ensures the Vec has capacity for n elements on top of len
-                // so we should reserve buffer_size - len. len will always be zero at this point
-                // this is a no-op on the first call as buffer_size == capacity
-                process_information.reserve(buffer_size);
-
-                match NtQuerySystemInformation(
-                    SystemProcessInformation,
-                    process_information.as_mut_ptr() as *mut _,
-                    buffer_size as _,
-                    &mut cb_needed,
-                )
-                .ok()
-                {
-                    Ok(()) => break,
-                    Err(err) if err.code() == STATUS_INFO_LENGTH_MISMATCH.to_hresult() => {
-                        // GetNewBufferSize
-                        if cb_needed == 0 {
-                            buffer_size *= 2;
-                            continue;
-                        }
-                        // allocating a few more kilo bytes just in case there are some new process
-                        // kicked in since new call to NtQuerySystemInformation
-                        buffer_size = (cb_needed + (1024 * 10)) as usize;
-                        continue;
-                    }
-                    Err(_err) => {
-                        sysinfo_debug!(
-                            "Couldn't get process infos: NtQuerySystemInformation returned {}",
-                            _err,
-                        );
-                        return 0;
-                    }
-                }
+            if let Err(_error) = Process32FirstW(*snapshot, &mut process_entry) {
+                sysinfo_debug!("Process32FirstW has failed: {_error:?}");
+                return 0;
             }
-
-            // If we reach this point NtQuerySystemInformation succeeded
-            // and the buffer contents are initialized
-            process_information.set_len(buffer_size);
-
-            let nb_updated = AtomicUsize::new(0);
-
-            // Parse the data block to get process information
-            let mut process_ids = Vec::with_capacity(500);
-            let mut process_information_offset = 0;
-            loop {
-                let p = process_information
-                    .as_ptr()
-                    .offset(process_information_offset)
-                    as *const SYSTEM_PROCESS_INFORMATION;
-
-                // read_unaligned is necessary to avoid
-                // misaligned pointer dereference: address must be a multiple of 0x8 but is 0x...
-                // under x86_64 wine (and possibly other systems)
-                let pi = ptr::read_unaligned(p);
-
-                if filter_callback(Pid(pi.UniqueProcessId as _), filter_array) {
-                    process_ids.push(Wrap(p));
-                }
-
-                if pi.NextEntryOffset == 0 {
-                    break;
-                }
-
-                process_information_offset += pi.NextEntryOffset as isize;
-            }
-            let process_list = Wrap(UnsafeCell::new(&mut self.process_list));
-            let nb_cpus = if refresh_kind.cpu() {
-                self.cpus.len() as u64
-            } else {
-                0
-            };
-
-            let now = get_now();
-
-            #[cfg(feature = "multithread")]
-            use rayon::iter::ParallelIterator;
-
-            // TODO: instead of using parallel iterator only here, would be better to be
-            //       able to run it over `process_information` directly!
-            let processes = into_iter(process_ids)
-                .filter_map(|pi| {
-                    nb_updated.fetch_add(1, Ordering::Relaxed);
-                    // as above, read_unaligned is necessary
-                    let pi = ptr::read_unaligned(pi.0);
-                    let pid = Pid(pi.UniqueProcessId as _);
-                    let ppid: usize = pi.InheritedFromUniqueProcessId as _;
-                    let parent = if ppid != 0 {
-                        Some(Pid(pi.InheritedFromUniqueProcessId as _))
-                    } else {
-                        None
-                    };
-                    // Not sure why we need to make this
-                    let process_list: &Wrap<UnsafeCell<&mut HashMap<Pid, Process>>> = &process_list;
-                    if let Some(proc_) = (*process_list.0.get()).get_mut(&pid) {
-                        let proc_ = &mut proc_.inner;
-                        if proc_
-                            .get_start_time()
-                            .map(|start| start == proc_.start_time())
-                            .unwrap_or(true)
-                        {
-                            proc_.update(refresh_kind, nb_cpus, now, false, &pi);
-                            // Update the parent in case it changed.
-                            proc_.parent = parent;
-                            return None;
-                        }
-                        // If the PID owner changed, we need to recompute the whole process.
-                        sysinfo_debug!("owner changed for PID {}", pid);
-                    }
-                    let name = get_process_name(&pi, pid);
-                    let mut p = ProcessInner::new(pid, parent, now, name);
-                    p.update(refresh_kind, nb_cpus, now, false, &pi);
-                    Some(Process { inner: p })
-                })
-                .collect::<Vec<_>>();
-            for p in processes.into_iter() {
-                self.process_list.insert(p.pid(), p);
-            }
-            nb_updated.into_inner()
         }
+
+        // Iterate over processes in the snapshot.
+        // Use Process32NextW to process the next PROCESSENTRY32W in the snapshot
+        loop {
+            let proc_id = Pid::from_u32(process_entry.th32ProcessID);
+
+            if filter_callback(proc_id, filter_array) {
+                // exists already
+                if let Some(p) = process_list.get_mut(&proc_id) {
+                    // Update with the most recent information
+                    let p = &mut p.inner;
+                    p.update(refresh_kind, nb_cpus, now, false);
+
+                    // Update parent process
+                    let parent = if process_entry.th32ParentProcessID == 0 {
+                        None
+                    } else {
+                        Some(Pid::from_u32(process_entry.th32ParentProcessID))
+                    };
+
+                    p.parent = parent;
+                } else {
+                    // Make a new 'ProcessInner' using the Windows PROCESSENTRY32W struct.
+                    let mut p = ProcessInner::from_process_entry(&process_entry, now);
+                    p.update(refresh_kind, nb_cpus, now, false);
+                    process_list.insert(proc_id, Process { inner: p });
+                }
+
+                num_procs += 1;
+            }
+
+            // nothing else to process
+            if unsafe { Process32NextW(*snapshot, &mut process_entry).is_err() } {
+                break;
+            }
+        }
+
+        num_procs
     }
 
     pub(crate) fn processes(&self) -> &HashMap<Pid, Process> {
@@ -360,10 +319,6 @@ impl SystemInner {
         self.cpus.cpus()
     }
 
-    pub(crate) fn physical_core_count(&self) -> Option<usize> {
-        get_physical_core_count()
-    }
-
     pub(crate) fn total_memory(&self) -> u64 {
         self.mem_total
     }
@@ -462,6 +417,15 @@ impl SystemInner {
     pub(crate) fn distribution_id() -> String {
         std::env::consts::OS.to_owned()
     }
+
+    pub(crate) fn distribution_id_like() -> Vec<String> {
+        Vec::new()
+    }
+
+    pub(crate) fn kernel_name() -> Option<&'static str> {
+        Some("Windows")
+    }
+
     pub(crate) fn cpu_arch() -> Option<String> {
         unsafe {
             // https://docs.microsoft.com/fr-fr/windows/win32/api/sysinfoapi/ns-sysinfoapi-system_info
@@ -486,6 +450,25 @@ impl SystemInner {
             }
         }
     }
+
+    pub(crate) fn physical_core_count() -> Option<usize> {
+        get_physical_core_count()
+    }
+
+    pub(crate) fn open_files_limit() -> Option<usize> {
+        // Apparently when using C run-time libraries, it's limited by _NHANDLE_.
+        // It's a define:
+        //
+        // ```
+        // #define IOINFO_L2E          6
+        // #define IOINFO_ARRAY_ELTS   (1 << IOINFO_L2E)
+        // #define IOINFO_ARRAYS       128
+        // #define _NHANDLE_ (IOINFO_ARRAYS * IOINFO_ARRAY_ELTS)
+        // ```
+        //
+        // So 128 * (1 << 6) = 8192
+        Some(8192)
+    }
 }
 
 pub(crate) fn is_proc_running(handle: HANDLE) -> bool {
@@ -494,41 +477,13 @@ pub(crate) fn is_proc_running(handle: HANDLE) -> bool {
         && exit_code == STILL_ACTIVE.0 as u32
 }
 
-#[allow(clippy::size_of_in_element_count)]
-//^ needed for "name.Length as usize / std::mem::size_of::<u16>()"
-pub(crate) fn get_process_name(process: &SYSTEM_PROCESS_INFORMATION, process_id: Pid) -> OsString {
-    let name = &process.ImageName;
-    if name.Buffer.is_null() {
-        match process_id.0 {
-            0 => "Idle".to_owned(),
-            4 => "System".to_owned(),
-            _ => format!("<no name> Process {process_id}"),
-        }
-        .into()
-    } else {
-        unsafe {
-            let slice = std::slice::from_raw_parts(
-                name.Buffer,
-                // The length is in bytes, not the length of string
-                name.Length as usize / std::mem::size_of::<u16>(),
-            );
-
-            OsString::from_wide(slice)
-        }
-    }
-}
-
 fn get_dns_hostname() -> Option<String> {
     let mut buffer_size = 0;
     // Running this first to get the buffer size since the DNS name can be longer than MAX_COMPUTERNAME_LENGTH
     // setting the `lpBuffer` to null will return the buffer size
     // https://docs.microsoft.com/en-us/windows/win32/api/sysinfoapi/nf-sysinfoapi-getcomputernameexw
     unsafe {
-        let _err = GetComputerNameExW(
-            ComputerNamePhysicalDnsHostname,
-            PWSTR::null(),
-            &mut buffer_size,
-        );
+        let _err = GetComputerNameExW(ComputerNamePhysicalDnsHostname, None, &mut buffer_size);
 
         // Setting the buffer with the new length
         let mut buffer = vec![0_u16; buffer_size as usize];
@@ -536,7 +491,7 @@ fn get_dns_hostname() -> Option<String> {
         // https://docs.microsoft.com/en-us/windows/win32/api/sysinfoapi/ne-sysinfoapi-computer_name_format
         if GetComputerNameExW(
             ComputerNamePhysicalDnsHostname,
-            PWSTR::from_raw(buffer.as_mut_ptr()),
+            Some(PWSTR::from_raw(buffer.as_mut_ptr())),
             &mut buffer_size,
         )
         .is_ok()
@@ -585,13 +540,15 @@ struct RegKey(HKEY);
 impl RegKey {
     unsafe fn open(hkey: HKEY, path: &[u16]) -> Option<Self> {
         let mut new_hkey = Default::default();
-        if RegOpenKeyExW(
-            hkey,
-            PCWSTR::from_raw(path.as_ptr()),
-            0,
-            KEY_READ,
-            &mut new_hkey,
-        )
+        if unsafe {
+            RegOpenKeyExW(
+                hkey,
+                PCWSTR::from_raw(path.as_ptr()),
+                Some(0),
+                KEY_READ,
+                &mut new_hkey,
+            )
+        }
         .is_err()
         {
             return None;
@@ -607,14 +564,16 @@ impl RegKey {
     ) -> windows::core::Result<()> {
         let mut buf_type = REG_NONE;
 
-        RegQueryValueExW(
-            self.0,
-            PCWSTR::from_raw(field_name.as_ptr()),
-            None,
-            Some(&mut buf_type),
-            Some(buf.as_mut_ptr()),
-            Some(buf_len),
-        )
+        unsafe {
+            RegQueryValueExW(
+                self.0,
+                PCWSTR::from_raw(field_name.as_ptr()),
+                None,
+                Some(&mut buf_type),
+                Some(buf.as_mut_ptr()),
+                Some(buf_len),
+            )
+        }
         .ok()
     }
 }
